; -------------------------------------------------------------------------
;
;       This is an amendment to Skip Rowlands original 
;       getrecrd.sc
;
;       We originally needed this routine because of L11 problems,
;       and had to add the modifications for more conditions.
;       BTW If you have L11 problems, you might try increasing RAM or 
;       removing memory drivers.
;
;       This routine runs very slowly ......
;
;       There are "features" which need to be understood:
;
;       1.   If there is no image on desktop when trying
;            to use getrecord.l this will result in an error
;           You can make mods for this error but we have not made any
;           changes just yet.
;
;       2.  Calling getrecord.l when there is a key violation is a problem.
;           Get around by checking for keyviol before calling getrecord.l
;
;       If you have any improvements please let us know
;       Contact: 
;       Arnold Khoo / Ark Infoware 
;       Tel:  Singapore (65) 2239942
;       Email: CIS 75540,2404   or arkinfo@singnet.com.sg
;      -------------------------------------------------------------------------

Proc GetRecord.L(Table.A, Criteria.A, Dynarray.L, VarName.A)
    PRIVATE 

    ; **** changes made by Ark Infoware
    IsDetlForm.A,                               ;--- Included to check current form


    Curwindow.H,CurTable.A,CurField.A,CurRec.N, ;--- Current workspace marker
    OnWorkSpace.L,      ;--- Flag whether target table is on workspace
    Ct,                 ;--- Counter for number of fields with criteria
    Pinch.A,            ;--- Criteria value "pinched off Criteria.A String
    V1,V2,V3,V4,V5,V6,V7,V8,V9,V10,V11,V12,V13, ;--- poss. vars assigned locate criteria
    FieldNames.R, FieldTypes.R,  ;--- structure of Table.A (using TableInfo Fields)
    CurRecordLock.L,    ;--- Checking whether record is locked.
    Locate.E            ;--- locate string to execute
    Echo Off     

    ; **** changes made by Ark Info
    ; **** all arrays passed via varname will be initialized
    If Dynarray.L then
	Execute "DynArray "+VarName.A+"[]"
    Else
	Execute "Array "+VarName.A
    EndIf    
    ; ******************************************************

    ; Mark the current workspace
    If Nimages() > 0 Then 
       CurTable.A = Table()
       CurWindow.H= GetWindow()
       CurField.A = Field()
       CurRecordLock.L = False
       CurRec.N = RecNo()

       ; **** changes made by Ark Info
       If SysMode() = "CoEdit" And RecordStatus("Locked") Then ;--- for checking whether current
	  CurRecordLock.L = True      ;--- record is locked; once relocate
	  PostRecord                  ;--- wld relock again if true.
       Endif  
       IsDetlForm.A = False
       If IsFormView() Then 
	  While FormType("Detail") 
		UpImage
	  Endwhile
	  IsDetlForm.A = True
	  FormKey
       Endif
       ; ******************************************************
    Endif

    OnWorkSpace.L = IsOnWorkSpace(Table.A)
    ;--- this utility moves focus to the target table     
    ;--- if table is not on the workSpace, it will place it without a window
    MoveTable.U(Table.A,"N")  
    
    ;--- locate string is populated by values corresponding to the field type,
    ;--- from left to right, until all criteria has been assigned
    CtrlHome            
    Ct=0   Locate.E = "Locate "   CriteriaError.L=False
    TableInfo Table.A Fields To FieldNames.R, FieldTypes.R 
    If Search("",Criteria.A) <> 0 Then
       While Match(Criteria.A,"....",Pinch.A,Criteria.A)
	     Ct=Ct+1
	     L = AssignCriteria.L(Ct,FieldTypes.R[Ct],Pinch.A)
	     If Not L Then 
		CriteriaError.L = True
		QuitLoop
	     Endif
	     Locate.E = Locate.E+"v"+Strval(Ct)+","
       Endwhile
       Ct=Ct+1
       L = AssignCriteria.L(Ct,FieldTypes.R[Ct],Criteria.A)
       If Not L Then 
	  CriteriaError.L = True
       Endif
       Locate.E = Locate.E+"v"+Strval(Ct)
    Else
       Ct=1
       L = AssignCriteria.L(Ct,FieldTypes.R[Ct],Criteria.A)
       If Not L Then 
	  CriteriaError.L = True
       Endif
       Locate.E = Locate.E+"v1"     
    Endif 
    If Not CriteriaError.L Then
       CtrlHome
       If Ct=1 Then
	  Right
       Endif
    Endif
    Execute Locate.E
    If Dynarray.L Then 
       Execute "CopyToDynarray "+VarName.A
    Else
       Execute "CopyToArray "+VarName.A
    Endif
    Retval.L = Retval
    
    ;Start restoring the workspace
    If Not OnWorkSpace.L And Sysmode() <> "CoEdit" Then 
       ClearImage
    Endif
    If IsAssigned(CurWindow.H) Then 
       ; **** changes made by Ark Info
       If IsDetlForm.A Then 
	  If CurWindow.H <> 0 Then 
	     Window Select CurWindow.H        
	  Endif
       Endif
       ; ******************************************************

       Moveto CurTable.A       
       Moveto Field CurField.A
       Moveto Record CurRec.N

       ; **** changes made by Ark Info
       If CurRecordLock.L Then 
	  LockRecord
       Endif
       ; ******************************************************
    Endif
    If Not Retval.L Or CriteriaError.l Then 
       Execute "Release Vars "+VarName.A
       Return False
    Endif
    Release Vars CriteriaError.L
    Return True
Endproc
Writelib C1 GetRecord.L
Release Procs GetRecord.L

Proc MoveTable.U(Table.A,Action.A)
     ;--- a utility either placing a tbl on workspace or 
     ;--- moving focus to a tbl already on workspace
     ;--- place Table.A if not onworkspace: Action.A = V Or T Or N
     ;--- moveto Table.A if onworkspace   : Action.A = V Or T Or is blank
     ;--- move Table.A out of sight       : Action.A = V Or is blank
     ;--- clear Table.A from workspace    : Action = C
     ;--- the Utility populates a workspace mgtmt dynarray, WinHandle.Y with
     ;--- the handles of "windowed" tableviews. (the 'tv' in the assignment
     ;--- indicates a tableview)
     PRIVATE
     Wa,       ;--- dynarray for reseting window attributes
     OnTable.L ;--- flag set if target table is already on the workspace
     ;Winhandle.Y collects window handles
     If Not IsAssigned(WinHandle.Y) Then
	DynArray WinHandle.Y[]
     Endif
     If IsFieldView() Then 
	Formkey
     Endif
     If IsOnWorkSpace(Table.A) Then
	Moveto Table.A 
	OnTable.L = True
     Endif
     If IsAssigned(OnTable.L) And Action.A="C" Then
	ClearImage
	Release Vars WinHandle.Y[Table.A+".tv"],WinHandle.Y[Table.A+"fv"]
	Return
     Endif
     ;If not on Table.A Then need to place table.a
     If Not IsAssigned(OnTable.L) Then
	Switch
	    Case Action.A = "V" or Action.A = "T":
		 If SysMode() = "CoEdit" Then
		    Menu {TableAdd} Select Table.A
		 Else
		    View Table.A
		 Endif
		 WinHandle.Y[Table.A+".tv"] = GetWindow()
		 If Action.A = "T" Then  ;---don't move the tableview window
		    Return
		 Endif
		 Ontable.L = True
	    Case Action.A = "N" :
		 If SysMode() = "CoEdit" Then
		    Menu {TableAdd} Select Table.A
		    Window Destroy
		 Else
		    View NoWindow Table.A
		 Endif
		 Return
	Endswitch
     Endif
     ; hide the tableview window, since there is no way to tell if you are on a 
     ; windowless table, operate under the assumption that both WinHandle.Y and 
     ; OnTable.L must be assigned for the current table to have a window. If not 
     ; there is no window.
     If Not IsAssigned(WinHandle.Y[Table.A+".tv"]) or 
	Not IsAssigned(OnTable.L) Then
	Return
     Endif
     If Not IsWindow(WinHandle.Y[Table.A+".tv"]) Then
	Release Vars WinHandle.Y[Table.A+".tv"]
	Return
     Endif
     DynArray Wa[]
     Wa["OriginRow"] = IIf(Action.A = "T",0,-200)
     Window SetAttributes WinHandle.Y[Table.A+".tv"] From Wa
Endproc
Writelib C1 MoveTable.U
Release Procs MoveTable.U

Proc AssignCriteria.L(Var.S,FieldType.A,Value.A)
     ; Make var assignments based on field type
     Switch
	 Case Substr(FieldType.A,1,1) = "N" Or Substr(FieldType.A,1,1) = "S":
	      If Numval(Value.A) = "Error" Then 
		 Return False
	      Endif
	      VarAssign "v"+Strval(Var.S) Numval(Value.A)
	 Case Substr(FieldType.A,1,1) = "D" :
	      If Numval(Value.A) = "Error" Then 
		 Return False
	      Endif
	      VarAssign "v"+Strval(Var.S) DateVal(Value.A)
	 Otherwise:
	      VarAssign "v"+Strval(Var.S) Value.A
     Endswitch
     Return True
Endproc
Writelib C1 AssignCriteria.L
Release Procs AssignCriteria.L
